import os
from pathlib import Path
from typing import Dict, List, Tuple

try:
    from typing import TypedDict
except:
    from typing_extensions import TypedDict

import datasets

from seacrowd.sea_datasets.indocoref.utils.text_preprocess import \
    TextPreprocess
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{artari-etal-2021-multi,
  title        = {A Multi-Pass Sieve Coreference Resolution for {I}ndonesian},
  author       = {Artari, Valentina Kania Prameswara  and Mahendra, Rahmad  and Jiwanggi, Meganingrum Arista  and Anggraito, Adityo  and Budi, Indra},
  year         = 2021,
  month        = sep,
  booktitle    = {Proceedings of the International Conference on Recent Advances in Natural Language Processing (RANLP 2021)},
  publisher    = {INCOMA Ltd.},
  address      = {Held Online},
  pages        = {79--85},
  url          = {https://aclanthology.org/2021.ranlp-1.10},
  abstract     = {Coreference resolution is an NLP task to find out whether the set of referring expressions belong to the same concept in discourse. A multi-pass sieve is a deterministic coreference model that implements several layers of sieves, where each sieve takes a pair of correlated mentions from a collection of non-coherent mentions. The multi-pass sieve is based on the principle of high precision, followed by increased recall in each sieve. In this work, we examine the portability of the multi-pass sieve coreference resolution model to the Indonesian language. We conduct the experiment on 201 Wikipedia documents and the multi-pass sieve system yields 72.74{\%} of MUC F-measure and 52.18{\%} of BCUBED F-measure.}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indocoref"
_DESCRIPTION = """\
Dataset contains articles from Wikipedia Bahasa Indonesia which fulfill these conditions:
- The pages contain many noun phrases, which the authors subjectively pick: (i) fictional plots, e.g., subtitles for films,
  TV show episodes, and novel stories; (ii) biographies (incl. fictional characters); and (iii) historical events or important events.
- The pages contain significant variation of pronoun and named-entity. We count the number of first, second, third person pronouns,
  and clitic pronouns in the document by applying string matching.We examine the number
of named-entity using the Stanford CoreNLP
NER Tagger (Manning et al., 2014) with a
model trained from the Indonesian corpus
taken from Alfina et al. (2016).
The Wikipedia texts have length of 500 to
2000 words.
We sample 201 of pages from subset of filtered
Wikipedia pages. We hire five annotators who are
undergraduate student in Linguistics department.
They are native in Indonesian. Annotation is carried out using the Script d’Annotation des Chanes
de Rfrence (SACR), a web-based Coreference resolution annotation tool developed by Oberle (2018).
From the 201 texts, there are 16,460 mentions
tagged by the annotators
"""

_HOMEPAGE = "https://github.com/valentinakania/indocoref/"
_LICENSE = "MIT"
_URLS = {
    _DATASETNAME: "https://github.com/valentinakania/indocoref/archive/refs/heads/main.zip",
}
_SUPPORTED_TASKS = [Tasks.COREFERENCE_RESOLUTION]
# Does not seem to have versioning
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class Indocoref(datasets.GeneratorBasedBuilder):
    """A collection of 210 curated articles from Wikipedia Bahasa Indonesia with Coreference Annotations"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indocoref_source",
            version=SOURCE_VERSION,
            description="Indocoref source schema",
            schema="source",
            subset_id="indocoref",
        ),
        SEACrowdConfig(
            name="indocoref_seacrowd_kb",
            version=SEACROWD_VERSION,
            description="Indocoref Nusantara schema",
            schema="seacrowd_kb",
            subset_id="indocoref",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indocoref_source"

    def _info(self) -> datasets.DatasetInfo:
        # The dataset does not really come with a schema, the features here come from the returned value
        # of the accompanying utils files.
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int64"),
                    "passage": datasets.Value("string"),
                    "mentions": [
                        {
                            "id": datasets.Value("int64"),
                            # Two entities which share a label are coreferences
                            "labels": datasets.Sequence(datasets.Value("string")),
                            "class": datasets.Value("string"),
                            "text": datasets.Value("string"),
                            "pronoun": datasets.Value("bool"),
                            "proper": datasets.Value("bool"),
                            "sent": datasets.Value("int32"),
                            "cluster": datasets.Value("int32"),
                            "per": datasets.Value("bool"),
                            "org": datasets.Value("bool"),
                            "loc": datasets.Value("bool"),
                            "ner": datasets.Value("bool"),
                            # "offset" is only available after modifying the original util class
                            # "offset": datasets.Sequence(datasets.Value("int32"))
                            # POS tags were originally available but removed due to polyglot icu dependency
                            # polyglot.Text(passage, hint_language_code='id')
                        }
                    ],
                }
            )
        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    class ReadPassage(TypedDict):
        passage: str
        annotated: str
        mentions: List[any]

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        base_path = Path(dl_manager.download_and_extract(urls)) / "indocoref-main" / "data"
        passage_path = base_path / "passage"
        annotated_path = base_path / "annotated"
        mentions_per_file = TextPreprocess(annotated_path).run(0)

        data: List[self.ReadPassage] = []
        for passage_file_name, annotated_file_name in zip(sorted(os.listdir(passage_path)), sorted(os.listdir(annotated_path))):
            passage_file_path, annotated_file_path = passage_path / passage_file_name, annotated_path / annotated_file_name

            if os.path.isfile(passage_file_path) and os.path.isfile(annotated_file_path):
                with open(passage_file_path, "r") as fpassage, open(annotated_file_path, "r") as fannotated:
                    data.append(self.ReadPassage(passage=fpassage.read(), annotated=fannotated.read(), mentions=mentions_per_file[annotated_file_name]))

        # Dataset has no predefined splits, using datasets.Split.TRAIN for all of the data.
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "data": data,
                    "split": "train",
                },
            ),
        ]

    class DisjointSet:
        parent = {}

        def __init__(self, items):
            for item in items:
                self.parent[item] = item

        def find(self, k):
            if self.parent[k] == k:
                return k
            return self.find(self.parent[k])

        def union(self, a, b):
            x = self.find(a)
            y = self.find(b)
            self.parent[x] = y

    def _generate_examples(self, data: List[ReadPassage], split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        if self.config.schema == "source":
            for index, example in enumerate(data):
                passage, mentions = example["passage"], example["mentions"]
                row = {
                    "id": index,
                    "passage": passage,
                    "mentions": [
                        {
                            "id": mention["id"],
                            "labels": mention["labels"],
                            "class": mention["class"],
                            "text": mention["text"],
                            "pronoun": mention["pronoun"],
                            "proper": mention["proper"],
                            "sent": mention["sent"],
                            "cluster": mention["cluster"],
                            "per": mention["per"],
                            "org": mention["org"],
                            "loc": mention["loc"],
                            "ner": mention["ner"],
                        }
                        for mention in mentions
                    ],
                }
                yield index, row

        elif self.config.schema == "seacrowd_kb":
            for index, example in enumerate(data):
                passage, mentions = example["passage"], example["mentions"]
                # Annotated text does not have any line breaks but the original passage does
                passage = passage.replace(" \n", " ")
                passage = passage.replace("\n", " ")
                all_labels = {label for mention in mentions for label in mention["labels"]}
                labels_disjoint_set = self.DisjointSet(all_labels)
                for mention in mentions:
                    for i in range(1, len(mention["labels"])):
                        labels_disjoint_set.union(mention["labels"][i], mention["labels"][i - 1])
                coreferences = {}
                for mention in mentions:
                    coreference_id = labels_disjoint_set.find(mention["labels"][0])
                    if coreference_id not in coreferences:
                        coreferences[coreference_id] = []
                    coreferences[coreference_id].append(str(mention["id"]))

                row_id = str(index)
                row = {
                    "id": row_id,
                    "passages": [{"id": "passage-" + row_id, "type": "text", "text": [passage], "offsets": [[0, len(passage)]]}],
                    "entities": [
                        {
                            "id": row_id + "-entity-" + str(mention["id"]),
                            "type": mention["class"],
                            "text": [mention["text"]],
                            "offsets": [list(mention["offset"])],
                            "normalized": [],
                        }
                        for mention in mentions
                    ],
                    "coreferences": [{"id": row_id + "-coreference-" + str(coref_id), "entity_ids": [row_id + "-entity-" + entity_id for entity_id in entity_ids]} for coref_id, entity_ids in enumerate(coreferences.values())],
                    "events": [],
                    "relations": [],
                }
                yield index, row



"""This code is partially taken from https://github.com/huggingface/datasets/blob/main/datasets/xcopa/xcopa.py."""

import json

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_HOMEPAGE = "https://github.com/cambridgeltl/xcopa"

_CITATION = """\
@inproceedings{ponti2020xcopa,
  title={{XCOPA: A} Multilingual Dataset for Causal Commonsense Reasoning},
  author={Edoardo M. Ponti, Goran Glava\v{s}, Olga Majewska, Qianchu Liu, Ivan Vuli\'{c} and Anna Korhonen},
  booktitle={Proceedings of the 2020 Conference on Empirical Methods in Natural Language Processing (EMNLP)},
  year={2020},
  url={https://ducdauge.github.io/files/xcopa.pdf}
}
@inproceedings{roemmele2011choice,
  title={Choice of plausible alternatives: An evaluation of commonsense causal reasoning},
  author={Roemmele, Melissa and Bejan, Cosmin Adrian and Gordon, Andrew S},
  booktitle={2011 AAAI Spring Symposium Series},
  year={2011},
  url={https://people.ict.usc.edu/~gordon/publications/AAAI-SPRING11A.PDF},
}
"""

_LANGUAGES = ["ind", "tha", "vie"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "xcopa"

_DESCRIPTION = """\
  XCOPA: A Multilingual Dataset for Causal Commonsense Reasoning
The Cross-lingual Choice of Plausible Alternatives dataset is a benchmark to evaluate the ability of machine learning models to transfer commonsense reasoning across
languages. The dataset is the translation and reannotation of the English COPA (Roemmele et al. 2011) and covers 11 languages from 11 families and several areas around
the globe. The dataset is challenging as it requires both the command of world knowledge and the ability to generalise to new languages. All the details about the
creation of XCOPA and the implementation of the baselines are available in the paper.
"""

_HOMEPAGE = "https://github.com/cambridgeltl/xcopa"

_LICENSE = Licenses.CC_BY_4_0.value

_URLS = {
    "ind": [
        "https://raw.githubusercontent.com/cambridgeltl/xcopa/master/data/id/val.id.jsonl",
        "https://raw.githubusercontent.com/cambridgeltl/xcopa/master/data/id/test.id.jsonl",
    ],
    "tha": [
        "https://raw.githubusercontent.com/cambridgeltl/xcopa/master/data/th/val.th.jsonl",
        "https://raw.githubusercontent.com/cambridgeltl/xcopa/master/data/th/test.th.jsonl",
    ],
    "vie": [
        "https://raw.githubusercontent.com/cambridgeltl/xcopa/master/data/vi/val.vi.jsonl",
        "https://raw.githubusercontent.com/cambridgeltl/xcopa/master/data/vi/test.vi.jsonl",
    ],
}

_SUPPORTED_TASKS = [Tasks.COMMONSENSE_REASONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


def _xcopa_config_constructor(lang: str, schema: str, version: str) -> SEACrowdConfig:
    return SEACrowdConfig(
        name="xcopa_{}_{}".format(lang, schema),
        version=version,
        description="XCOPA {} schema".format(schema),
        schema=schema,
        subset_id="xcopa",
    )


class Xcopa(datasets.GeneratorBasedBuilder):
    """The Cross-lingual Choice of Plausible Alternatives dataset is a benchmark to evaluate the ability of machine learning models to transfer commonsense reasoning across
    languages. The dataset is the translation and reannotation of the English COPA (Roemmele et al. 2011) and covers 11 languages from 11 families and several areas around
    the globe."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [_xcopa_config_constructor(lang, "source", _SOURCE_VERSION) for lang in _LANGUAGES] + [_xcopa_config_constructor(lang, "seacrowd_qa", _SEACROWD_VERSION) for lang in _LANGUAGES]

    DEFAULT_CONFIG_NAME = "xcopa_ind_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "premise": datasets.Value("string"),
                    "choice1": datasets.Value("string"),
                    "choice2": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "label": datasets.Value("int32"),
                    "idx": datasets.Value("int32"),
                    "changed": datasets.Value("bool"),
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features_in_dict = features.to_dict()
            features_in_dict["meta"] = {"is_changed": {"dtype": "bool", "_type": "Value"}, "reasoning_type": {"dtype": "string", "_type": "Value"}}
            features = datasets.Features.from_dict(features_in_dict)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def get_lang(self, name: str):
        # xcopa_ind|
        # [xcopa, ind]
        names_splitted = name.split("_")
        if len(names_splitted) == 0:
            return "ind"
        return names_splitted[1]

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        urls = _URLS[self.get_lang(self.config.name)]
        data_dir = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir[0],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir[1],
                },
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        if self.config.schema == "source":
            with open(filepath, encoding="utf-8") as f:
                for row in f:
                    data = json.loads(row)
                    idx = data["idx"]
                    yield idx, data

        elif self.config.schema == "seacrowd_qa":
            with open(filepath, encoding="utf-8") as f:
                for row in f:
                    data = json.loads(row)
                    idx = data["idx"]
                    sample = {
                        "id": str(idx),
                        "question_id": str(idx),
                        "document_id": str(idx),
                        "question": "",
                        "type": "multiple_choice",
                        "choices": [data["choice1"], data["choice2"]],
                        "context": data["premise"],
                        "answer": [data["choice1"] if data["label"] == 0 else data["choice2"]],
                        "meta": {"is_changed": data["changed"], "reasoning_type": data["question"]},
                    }
                    yield idx, sample

        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses
from seacrowd.utils import schemas

_CITATION = """\
@mastersthesis{chumpolsathien_2020,
    title={Using Knowledge Distillation from Keyword Extraction to Improve the Informativeness of Neural Cross-lingual Summarization},
    author={Chumpolsathien, Nakhun},
    year={2020},
    school={Beijing Institute of Technology}
"""

_DATASETNAME = "thai_sum"

_DESCRIPTION = """
We present ThaiSum, a large-scale corpus for Thai text summarization obtained from several online news websites namely Thairath, ThaiPBS, Prachathai, and The Standard. This dataset consists of over 350,000 article and summary pairs written by journalists.
"""

_HOMEPAGE = "https://github.com/nakhunchumpolsathien/ThaiSum"

_LICENSE = Licenses.MIT.value

_LANGUAGES = ["tha"]

_LOCAL = False

_URLS = {
    "train": "https://nakhun-chumpolsathien.oss-us-west-1.aliyuncs.com/thaisum/thaisum.csv",
    "val": "https://nakhun-chumpolsathien.oss-us-west-1.aliyuncs.com/thaisum/validation_set.csv",
    "test": "https://nakhun-chumpolsathien.oss-us-west-1.aliyuncs.com/thaisum/test_set.csv",
}

_SUPPORTED_TASKS = [Tasks.SUMMARIZATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class ThaiSumDataset(datasets.GeneratorBasedBuilder):
    """
    Sequence-to-sequence (Seq2Seq) models have shown great achievement in text summarization.
    However, Seq2Seq model often requires large-scale training data to achieve effective results.
    Although many impressive advancements in text summarization field have been made,
    most of summarization studies focus on resource-rich languages.
    The progress of Thai text summarization is still far behind.
    The dearth of large-scale dataset keeps Thai text summarization in its infancy.
    As far as our knowledge goes, there is not a large-scale dataset for Thai text summarization available anywhere.
    Thus, we present ThaiSum, a large-scale corpus for Thai text summarization
    obtained from several online news websites namely Thairath, ThaiPBS, Prachathai, and The Standard.
    This dataset consists of over 350,000 article and summary pairs written by journalists.
    We evaluate the performance of various existing summarization models on ThaiSum dataset and analyse
    the characteristic of the dataset to present its difficulties.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"title": datasets.Value("string"), "body": datasets.Value("string"), "summary": datasets.Value("string"), "type": datasets.Value("string"), "tags": datasets.Value("string"), "url": datasets.Value("string")})
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        path_dict = dl_manager.download_and_extract(_URLS)
        train_path, val_path, test_path = path_dict["train"], path_dict["val"], path_dict["test"]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": val_path,
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        csv.field_size_limit(int(1000000))
        with open(filepath, encoding="utf-8") as f:
            csv_reader = csv.reader(f)
            next(csv_reader)  # skip header
            if self.config.schema == "source":
                for id_, row in enumerate(csv_reader):
                    yield id_, {
                        "title": row[0],
                        "body": row[1],
                        "summary": row[2],
                        "type": row[3],
                        "tags": row[4],
                        "url": row[5],
                    }
            elif self.config.schema == "seacrowd_t2t":
                for id_, row in enumerate(csv_reader):
                    yield id_, {
                        "id": str(id_),
                        "text_1": row[1],
                        "text_2": row[2],
                        "text_1_name": "document",
                        "text_2_name": "summary",
                    }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks


_CITATION = """\
@inproceedings{sakti-icslp-2004,
    title = "Indonesian Speech Recognition for Hearing and Speaking Impaired People",
    author = "Sakti, Sakriani and Hutagaol, Paulus and Arman, Arry Akhmad and Nakamura, Satoshi",
    booktitle = "Proc. International Conference on Spoken Language Processing (INTERSPEECH - ICSLP)",
    year = "2004",
    pages = "1037--1040"
    address = "Jeju Island, Korea"
}
"""

_DATASETNAME = "indspeech_teldialog_svcsr"

_DESCRIPTION = """\
This is the first Indonesian speech dataset for small vocabulary continuous speech recognition (SVCSR).
The data was developed by TELKOMRisTI (R&D Division, PT Telekomunikasi Indonesia) in collaboration with Advanced
Telecommunication Research Institute International (ATR) Japan and Bandung Institute of Technology (ITB) under the
Asia-Pacific Telecommunity (APT) project in 2004 [Sakti et al., 2004]. Although it was originally developed for
a telecommunication system for hearing and speaking impaired people, it can be used for other applications,
i.e., automatic call centers. Furthermore, as all speakers utter the same sentences,
it can also be used for voice conversion tasks.

The text is based on a word vocabulary which is derived from some necessary dialog calls,
such as dialog calls with the 119 emergency department, 108 telephone information department,
and ticket reservation department. In total, it consists of 20,000 utterances (about 18 hours of speech) from the
70-word dialog vocabulary of 100 sentences (including single word sentences) each uttered by 200 speakers
(100 Females, 100 Males). The age is limited to middle age (20-40 years), but they present a wide range of spoken
dialects from different ethnic groups. The recording is conducted in parallel for both clean and telephone speech,
but we open only the clean speech due to quality issues on telephone speech.
Each audio file is a single-channel 16-bit PCM WAV with a sample rate of 16000 Hz.
These utterances are equally split into training and test sets with 100 speakers (50 Females, 50 Males) in each set.
"""

_HOMEPAGE = "https://github.com/s-sakti/data_indsp_teldialog_svcsr/"

_LICENSE = "CC-BY-NC-SA-4.0"

_LANGUAGES = ["ind"]
_LOCAL = False

URL_TEMPLATE = "https://raw.githubusercontent.com/s-sakti/data_indsp_teldialog_svcsr/main/"
_URLS = {
    _DATASETNAME: {"lst": URL_TEMPLATE + "lst/", "speech": URL_TEMPLATE + "speech/", "text": URL_TEMPLATE + "text/"},
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class INDspeechTELDIALOGSVCSR(datasets.GeneratorBasedBuilder):
    """
    This is an Indonesian speech dataset on small vocabulary continuous speech recognition (SVCSR) from necessary
    dialog calls. The dataset loader is designed for speech recognition task.
    There are 20000 utterances (train: 10000, test:10000) uttered by 200 speakers (50 male 50 female each in train and
    test).
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indspeech_teldialog_svcsr_source",
            version=SOURCE_VERSION,
            description="indspeech_teldialog_svcsr source schema",
            schema="source",
            subset_id="indspeech_teldialog_svcsr",
        ),
        SEACrowdConfig(
            name="indspeech_teldialog_svcsr_seacrowd_sptext",
            version=SEACROWD_VERSION,
            description="indspeech_teldialog_svcsr Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="indspeech_teldialog_svcsr",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indspeech_teldialog_svcsr_source"

    def _info(self) -> datasets.DatasetInfo:

        # Create the source schema; this schema will keep all keys/information/labels as close to the original dataset as possible.

        # You can arbitrarily nest lists and dictionaries.
        # For iterables, use lists over tuples or `datasets.Sequence`

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "speaker_id": datasets.Value("string"),
                    "gender_id": datasets.Value("string"),
                    "utterance_id": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="sentences")],
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = {
            "spk_data": {"train": dl_manager.download_and_extract(os.path.join(urls["lst"], "train_spk.lst")), "test": dl_manager.download_and_extract(os.path.join(urls["lst"], "test_spk.lst"))},
            "wav_data": {"train": dl_manager.download_and_extract(os.path.join(urls["lst"], "train_wav.lst")), "test": dl_manager.download_and_extract(os.path.join(urls["lst"], "test_wav.lst"))},
            "txt_data": dl_manager.download_and_extract(os.path.join(urls["text"], "text.zip")),
        }
        speakers = {}
        with open(data_dir["spk_data"]["train"], "r") as f:
            speakers["train"] = [sp.replace("\n", "") for sp in f.readlines()]
        f.close()
        with open(data_dir["spk_data"]["test"], "r") as f:
            speakers["test"] = [sp.replace("\n", "") for sp in f.readlines()]
        f.close()
        data_dir["speech_path"] = {
            "train": {sp: dl_manager.download_and_extract(os.path.join(urls["speech"], "train", sp + ".zip")) for sp in speakers["train"]},
            "test": {sp: dl_manager.download_and_extract(os.path.join(urls["speech"], "test", sp + ".zip")) for sp in speakers["test"]},
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": data_dir["wav_data"]["train"],
                    "audio_path": data_dir["speech_path"]["train"],
                    "text_path": data_dir["txt_data"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["wav_data"]["test"],
                    "audio_path": data_dir["speech_path"]["test"],
                    "text_path": data_dir["txt_data"],
                    "split": "test",
                },
            ),
        ]

    @staticmethod
    def text_process(utterance_txt_dir):
        with open(utterance_txt_dir + ".ANS", "r") as f:
            lines = [x.replace("\n", "") for x in f.readlines()]
        f.close()
        return " ".join(lines)

    def _generate_examples(self, filepath: Path, audio_path, text_path: Path, split: str) -> Tuple[int, Dict]:
        with open(filepath, "r") as f:
            filelist = [x.replace("\n", "") for x in f.readlines()]
        f.close()
        for fn in filelist:
            speaker_id = fn[:3]
            gender_id = fn[:1]
            utterance_id = fn[4:8]
            _id = fn.replace(".wav", "")
            text = self.text_process(os.path.join(text_path, utterance_id))
            if self.config.schema == "source":
                yield _id, {
                    "speaker_id": speaker_id,
                    "gender_id": gender_id,
                    "utterance_id": utterance_id,
                    "audio": os.path.join(audio_path[speaker_id], fn),
                    "text": text,
                }

            elif self.config.schema == "seacrowd_sptext":
                yield _id, {
                    "id": _id,
                    "speaker_id": speaker_id,
                    "text": text,
                    "path": os.path.join(audio_path[speaker_id], fn),
                    "audio": os.path.join(audio_path[speaker_id], fn),
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": gender_id,
                    },
                }



"""
Text to Text Schema

Several tasks boil down to transforming 1 string into annother string, including:

- Translation
- Summarization
- Paraphrasing
"""
import datasets

features = datasets.Features(
    {
        "id": datasets.Value("string"),
        "text_1": datasets.Value("string"),
        "text_2": datasets.Value("string"),
        "text_1_name": datasets.Value("string"),
        "text_2_name": datasets.Value("string"),
    }
)



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
KoPI-CC corpus

[seacrowd_schema_name] = ssp
"""

import gzip
import json
from typing import List

import datasets
import zstandard as zstd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_DATASETNAME = "kopi_cc"
_LANGUAGES  = ["ind"]
_LOCAL = False
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME
_URL = "https://commoncrawl.org/"
_CITATION = """\
       @ARTICLE{2022arXiv220106642A,
       author = {{Abadji}, Julien and {Ortiz Suarez}, Pedro and {Romary}, Laurent and {Sagot}, Benoit},
        title = "{Towards a Cleaner Document-Oriented Multilingual Crawled Corpus}",
      journal = {arXiv e-prints},
     keywords = {Computer Science - Computation and Language},
         year = 2022,
        month = jan,
          eid = {arXiv:2201.06642},
        pages = {arXiv:2201.06642},
archivePrefix = {arXiv},
       eprint = {2201.06642},
 primaryClass = {cs.CL},
       adsurl = {https://ui.adsabs.harvard.edu/abs/2022arXiv220106642A},
      adsnote = {Provided by the SAO/NASA Astrophysics Data System}
}
@inproceedings{AbadjiOrtizSuarezRomaryetal.2021,
  author    = {Julien Abadji and Pedro Javier Ortiz Su{\'a}rez and Laurent Romary and Benoit Sagot},
  title     = {Ungoliant: An optimized pipeline for the generation of a very large-scale multilingual web corpus},
  series = {Proceedings of the Workshop on Challenges in the Management of Large Corpora (CMLC-9) 2021. Limerick, 12 July 2021 (Online-Event)},
  editor    = {Harald L{\"u}ngen and Marc Kupietz and Piotr Bański and Adrien Barbaresi and Simon Clematide and Ines Pisetta},
  publisher = {Leibniz-Institut f{\"u}r Deutsche Sprache},
  address   = {Mannheim},
  doi       = {10.14618/ids-pub-10468},
  url       = {https://nbn-resolving.org/urn:nbn:de:bsz:mh39-104688},
  pages     = {1 -- 9},
  year      = {2021},
  abstract  = {Since the introduction of large language models in Natural Language Processing, large raw corpora have played a crucial role in Computational Linguistics.},
  language  = {en}
}

"""

_DESCRIPTION = """\
    KoPI-CC (Korpus Perayapan Indonesia)-CC is Indonesian Only Extract from Common Crawl snapshots ,each snapshots get extracted using ungoliant and get extra "filtering" using deduplication technique

"""

_HOMEPAGE = "https://huggingface.co/datasets/munggok/KoPI-CC"

_LICENSE = "CC0"

_URLS = {
    "raw": "https://huggingface.co/datasets/munggok/KoPI-CC/resolve/main/{snapshot}/raw/id_meta_{index}.jsonl.zst",
    "dedup": "https://huggingface.co/datasets/munggok/KoPI-CC/resolve/main/{snapshot}/dedup/oscar-{index:012d}.json.gz",
    "neardup": "https://huggingface.co/datasets/munggok/KoPI-CC/resolve/main/{snapshot}/neardup/oscar-neardup-{index:012d}.json.gz",
    "neardup_clean": "https://huggingface.co/datasets/munggok/KoPI-CC/resolve/main/{snapshot}/neardup_clean/cleaned_oscar-neardup-{index:012d}.json.gz",
}


_N_SHARDS_PER_SNAPSHOT = {
    "2021_10": {"dedup": 132, "neardup": 120, "neardup_clean": 120},
    "2021_17": {"raw": 31, "dedup": 47, "neardup": 41, "neardup_clean": 41},
    "2021_21": {"raw": 63, "dedup": 37, "neardup": 33, "neardup_clean": 33},
    "2021_25": {"raw": 31, "dedup": 32, "neardup": 28, "neardup_clean": 28},
    "2021_31": {"raw": 35, "dedup": 47, "neardup": 42, "neardup_clean": 42},
    "2021_39": {"raw": 35, "dedup": 44, "neardup": 38, "neardup_clean": 38},
    "2021_43": {"raw": 35, "dedup": 44, "neardup": 39, "neardup_clean": 39},
    "2021_49": {"dedup": 31, "neardup": 28, "neardup_clean": 28},
    "2022_05": {"raw": 40, "dedup": 18, "neardup": 18, "neardup_clean": 35},
    "2022_21": {"raw": 40, "dedup": 42, "neardup": 37, "neardup_clean": 37},
    "2022_27": {"raw": 79, "dedup": 38, "neardup": 33, "neardup_clean": 33},
}

_SNAP_CONFIG = []
for m in list(_N_SHARDS_PER_SNAPSHOT.keys()):
    ka = list(_N_SHARDS_PER_SNAPSHOT[m].keys())
    conf = [m + "-" + a for a in ka]
    _SNAP_CONFIG.extend(conf)
_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_ALL_CONFIG = ["all-raw", "all-dedup", "all-neardup", "all-neardup_clean"] + _SNAP_CONFIG

_SOURCE_VERSION = "2018.12.01"

_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(snapshot, schema, version):
    """Construct SEACrowdConfig"""
    if schema != "source" and schema != "seacrowd_ssp":
        raise ValueError(f"Invalid schema: {schema}")

    if snapshot == "":
        raise ValueError(f"Snapshot is required. Choose one of these Snapshot: {_ALL_CONFIG}.")
    elif snapshot in _SNAP_CONFIG + _ALL_CONFIG:
        return SEACrowdConfig(
            name=f"{_DATASETNAME}_{snapshot}_{schema}",
            version=datasets.Version(version),
            description=f"KoPI-CC with {schema} schema for {snapshot}",
            schema=schema,
            subset_id="kopi_cc",
        )
    else:
        raise ValueError(f"Invalid language: {snapshot}. Choose one of these snapshots: {_ALL_CONFIG}.")


class KoPICC(datasets.GeneratorBasedBuilder):

    DEFAULT_CONFIG_NAME = "2021_17_dedup"

    BUILDER_CONFIGS = [seacrowd_config_constructor(sn, "source", _SOURCE_VERSION) for sn in _ALL_CONFIG] + [seacrowd_config_constructor(sn, "seacrowd_ssp", _SEACROWD_VERSION) for sn in _ALL_CONFIG]

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "timestamp": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "meta": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        name = self.config.name.replace("_" + self.config.schema, "")
        name = name.replace(_DATASETNAME + "_", "")
        split_name = name.split("-")
        if split_name[0] == "all":
            urls = []
            keys = list(_N_SHARDS_PER_SNAPSHOT.keys())
            idx = 0
            if split_name[1] == "raw":
                idx = 1
                keys = [ur for ur in list(_N_SHARDS_PER_SNAPSHOT.keys()) if _N_SHARDS_PER_SNAPSHOT[ur].get("raw") is not None]
            for m in keys:
                urls.extend([_URLS[split_name[1]].format(snapshot=m, index=k + idx) for k in range(_N_SHARDS_PER_SNAPSHOT[m].get(split_name[1]))])
        else:
            urls = [_URLS[split_name[1]].format(snapshot=split_name[0], index=k + 1) for k in range(_N_SHARDS_PER_SNAPSHOT[split_name[0]][split_name[1]])]
        path = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepaths": path, "split": "train", "type": split_name[1]},
            ),
        ]

    def _generate_examples(self, filepaths, split, type):
        """This function returns the examples in the raw (text) form by iterating on all the files."""
        id_ = 0
        for filepath in filepaths:
            if type == "raw":
                with zstd.open(open(filepath, "rb"), "rt", encoding="utf-8") as f:
                    for line in f:
                        if line:
                            example = json.loads(line)
                            meta = dict()
                            meta["warc_headers"] = example["warc_headers"]
                            meta["warc_headers"]["warc-identified-content-language"] = example["warc_headers"].get("warc-identified-content-language")
                            meta["identification"] = example["metadata"]["identification"]
                            meta["annotations"] = example["metadata"]["annotation"]
                            meta["line_identifications"] = example["metadata"]["sentence_identifications"]
                            if self.config.schema == "seacrowd_ssp":
                                yield id_, {"id": str(id_), "text": example["content"]}
                                id_ += 1
                            else:
                                yield id_, {"text": example["content"], "url": example["warc_headers"]["warc-target-uri"], "timestamp": example["warc_headers"]["warc-date"], "meta": json.dumps(meta)}
                                id_ += 1
            else:
                with gzip.open(open(filepath, "rb"), "rt", encoding="utf-8") as f:
                    for line in f:
                        if line:
                            example = json.loads(line)
                            if self.config.schema == "seacrowd_ssp":
                                yield id_, {"id": str(id_), "text": example["text"]}
                                id_ += 1
                            else:
                                yield id_, {"text": example["text"], "url": example["url"], "timestamp": example["timestamp"], "meta": example["meta"]}
                                id_ += 1



# coding=utf-8

import json
from pathlib import Path
import re
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{thanh-etal-2021-span,
    title = "Span Detection for Aspect-Based Sentiment Analysis in Vietnamese",
    author = "Thanh, Kim Nguyen Thi  and
      Khai, Sieu Huynh  and
      Huynh, Phuc Pham  and
      Luc, Luong Phan  and
      Nguyen, Duc-Vu  and
      Van, Kiet Nguyen",
    booktitle = "Proceedings of the 35th Pacific Asia Conference on Language, Information and Computation",
    year = "2021",
    publisher = "Association for Computational Lingustics",
    url = "https://aclanthology.org/2021.paclic-1.34",
    pages = "318--328",
}
"""

_DATASETNAME = "uit_visd4sa"

_DESCRIPTION = """\
This dataset is designed for span detection for aspect-based sentiment analysis NLP task.
A Vietnamese dataset consisting of 35,396 human-annotated spans on 11,122 feedback
comments for evaluating span detection for aspect-based sentiment analysis for mobile e-commerce
"""

_HOMEPAGE = "https://github.com/kimkim00/UIT-ViSD4SA"

_LICENSE = Licenses.UNKNOWN.value

_LANGUAGES = ["vie"]

_URLS = {
    "train": "https://raw.githubusercontent.com/kimkim00/UIT-ViSD4SA/main/data/train.jsonl",
    "dev": "https://raw.githubusercontent.com/kimkim00/UIT-ViSD4SA/main/data/dev.jsonl",
    "test": "https://raw.githubusercontent.com/kimkim00/UIT-ViSD4SA/main/data/test.jsonl",
}

_SUPPORTED_TASKS = [Tasks.SPAN_BASED_ABSA]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LOCAL = False


def construct_label_classes():
    IOB_tag = ["I", "O", "B"]
    aspects = ["SCREEN", "CAMERA", "FEATURES", "BATTERY", "PERFORMANCE", "STORAGE", "DESIGN", "PRICE", "GENERAL", "SER&ACC"]
    ratings = ["POSITIVE", "NEUTRAL", "NEGATIVE"]
    label_classes = []
    for iob in IOB_tag:
        if iob == "O":
            label_classes.append("O")
        else:
            for aspect in aspects:
                for rating in ratings:
                    label_classes.append("{iob}-{aspect}#{rating}".format(iob=iob, aspect=aspect, rating=rating))
    return label_classes


def construct_IOB_sequences(text, labels):
    labels.sort()
    word_start = [0] + [match.start() + 1 for match in re.finditer(" ", text)]
    is_not_O = False
    iob_sequence = []
    word_count = 0
    lb_count = 0

    while word_count < len(word_start):
        if lb_count == len(labels):
            for x in range(word_count, len(word_start)):
                iob_sequence.append("O")
            break
        if not is_not_O:
            if word_start[word_count] >= labels[lb_count][0]:
                is_not_O = True
                iob_sequence.append("B-" + labels[lb_count][-1])
                word_count += 1
            else:
                iob_sequence.append("O")
                word_count += 1
        else:
            if word_start[word_count] > labels[lb_count][1]:
                is_not_O = False
                lb_count += 1
            else:
                iob_sequence.append("I-" + labels[lb_count][-1])
                word_count += 1
    return iob_sequence


class UITViSD4SADataset(datasets.GeneratorBasedBuilder):
    """This dataset is designed for span detection for aspect-based sentiment analysis NLP task.
    A Vietnamese dataset consisting of 35,396 human-annotated spans on 11,122 feedback
    comments for evaluating span detection for aspect-based sentiment analysis for mobile e-commerce"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="uit_visd4sa source schema",
            schema="source",
            subset_id="uit_visd4sa",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description="uit_visd4sa SEACrowd schema",
            schema="seacrowd_seq_label",
            subset_id="uit_visd4sa",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.Sequence({"start": datasets.Value("int32"), "end": datasets.Value("int32"), "aspect": datasets.Value("string"), "rating": datasets.Value("string")}),
                }
            )

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(construct_label_classes())

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        path_dict = dl_manager.download_and_extract(_URLS)
        train_path, dev_path, test_path = path_dict["train"], path_dict["dev"], path_dict["test"]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": dev_path,
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        with open(filepath, "r") as f:
            df = [json.loads(line) for line in f.readlines()]
        f.close()
        if self.config.schema == "source":
            for _id, row in enumerate(df):
                labels = row["labels"]
                entry_labels = []
                for lb in labels:
                    entry_labels.append({"start": lb[0], "end": lb[1], "aspect": lb[-1].split("#")[0], "rating": lb[-1].split("#")[-1]})
                entry = {
                    "text": row["text"],
                    "label": entry_labels,
                }
                yield _id, entry
        elif self.config.schema == "seacrowd_seq_label":
            for _id, row in enumerate(df):
                entry = {
                    "id": str(_id),
                    "tokens": row["text"].split(" "),
                    "labels": construct_IOB_sequences(row["text"], row["labels"]),
                }
                yield _id, entry



import os
import pickle
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{
    ladhak-wiki-2020,
    title={WikiLingua: A New Benchmark Dataset for Multilingual Abstractive Summarization},
    author={Faisal Ladhak, Esin Durmus, Claire Cardie and Kathleen McKeown},
    booktitle={Findings of EMNLP, 2020},
    year={2020}
}
"""

_DATASETNAME = "wikilingua"

_DESCRIPTION = """\
We introduce WikiLingua, a large-scale, multilingual dataset for the evaluation of crosslingual abstractive 
summarization systems. We extract article and summary pairs in 18 languages from WikiHow12, a high quality, 
collaborative resource of how-to guides on a diverse set of topics written by human authors. We create gold-standard 
article summary alignments across languages by aligning the images that are used to describe each how-to step in an 
article.
"""

_HOMEPAGE = "https://github.com/esdurmus/Wikilingua"

_LANGUAGES = ["ind"]

_LICENSE = "CC-BY-NC-SA 3.0"

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://drive.google.com/u/0/uc?id=1PGa8j1_IqxiGTc3SU6NMB38sAzxCPS34&export=download"
}

_SUPPORTED_TASKS = [Tasks.SUMMARIZATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class Wikilingua(datasets.GeneratorBasedBuilder):
    """
    The dataset includes 47,511 articles from WikiHow. Extracted gold-standard article-summary alignments across
    languages by aligning the images that are used to describe each how-to step in an article.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="wikilingua_source",
            version=SOURCE_VERSION,
            description="wikilingua source schema",
            schema="source",
            subset_id="wikilingua",
        ),
        SEACrowdConfig(
            name="wikilingua_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="wikilingua Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="wikilingua",
        ),
    ]

    DEFAULT_CONFIG_NAME = "wikilingua_source"

    def _info(self) -> datasets.DatasetInfo:
        features = []
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int64"),
                    "link": datasets.Value("string"),
                    "main_point": datasets.Value("string"),
                    "summary": datasets.Value("string"),
                    "document": datasets.Value("string"),
                    "english_section_name": datasets.Value("string"),
                    "english_url": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            with open(filepath, "rb") as file:
                indonesian_docs = pickle.load(file)

            _id = 1
            for key_link, articles in indonesian_docs.items():
                for main_point, items in articles.items():
                    example = {"id": _id, "link": key_link, "main_point": main_point, "summary": items["summary"], "document": items["document"], "english_section_name": items["english_section_name"], "english_url": items["english_url"]}
                    yield _id, example
                    _id += 1
        elif self.config.schema == "seacrowd_t2t":
            with open(filepath, "rb") as file:
                indonesian_docs = pickle.load(file)

            _id = 1
            for key_link, articles in indonesian_docs.items():
                for main_point, items in articles.items():
                    example = {"id": _id, "text_1": items["document"], "text_2": items["summary"], "text_1_name": "document", "text_2_name": "summary"}
                    yield _id, example
                    _id += 1



import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{Artetxe:etal:2019,
      author    = {Mikel Artetxe and Sebastian Ruder and Dani Yogatama},
      title     = {On the cross-lingual transferability of monolingual representations},
      journal   = {CoRR},
      volume    = {abs/1910.11856},
      year      = {2019},
      archivePrefix = {arXiv},
      eprint    = {1910.11856}
}
"""

_DATASETNAME = "xquad"

_DESCRIPTION = """\
XQuAD (Cross-lingual Question Answering Dataset) is a benchmark dataset for evaluating cross-lingual question answering performance.
The dataset consists of a subset of 240 paragraphs and 1190 question-answer pairs from the development set of SQuAD v1.1 together (Rajpurkar et al., 2016)
with their professional translations into ten languages in their original implementation: Spanish, German, Greek, Russian, Turkish, Arabic, Vietnamese, Thai, Chinese, and Hindi and two in this dataloader: Vietnamese & Thai
"""

_HOMEPAGE = "https://github.com/google-deepmind/xquad"

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False
_LANGUAGES = ["tha", "vie"]

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class XQuADDataset(datasets.GeneratorBasedBuilder):
    """
    XQuAD (Cross-lingual Question Answering Dataset) is a benchmark dataset for evaluating cross-lingual question answering performance.
    The dataset consists of a subset of 240 paragraphs and 1190 question-answer pairs from the development set of SQuAD v1.1 together
    with their professional translations into ten languages: Spanish, German, Greek, Russian, Turkish, Arabic, Vietnamese, Thai, Chinese, and Hindi.
    """

    subsets = ["xquad.vi", "xquad.th"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="{sub}_source".format(sub=subset),
            version=datasets.Version(_SOURCE_VERSION),
            description="{sub} source schema".format(sub=subset),
            schema="source",
            subset_id="{sub}".format(sub=subset),
        )
        for subset in subsets
    ] + [
        SEACrowdConfig(
            name="{sub}_seacrowd_qa".format(sub=subset),
            version=datasets.Version(_SEACROWD_VERSION),
            description="{sub} SEACrowd schema".format(sub=subset),
            schema="seacrowd_qa",
            subset_id="{sub}".format(sub=subset),
        )
        for subset in subsets
    ]

    DEFAULT_CONFIG_NAME = "xquad.vi_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {"context": datasets.Value("string"), "question": datasets.Value("string"), "answers": datasets.Features({"answer_start": [datasets.Value("int64")], "text": [datasets.Value("string")]}), "id": datasets.Value("string")}
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN
            )
        ]

    def _generate_examples(self) -> Tuple[int, Dict]:
        name_split = self.config.name.split("_")
        subset_name = name_split[0]
        dataset = datasets.load_dataset(_DATASETNAME, subset_name)
        
        # Validation is the only subset name available for this dataset
        for data in dataset['validation']:
            if self.config.schema == "source":
                yield data['id'], {
                    "context": data['context'],
                    "question": data['question'],
                    "answers": {"answer_start": str(data['answers']['answer_start'][0]), "text": data['answers']['text'][0]},
                    "id": data['id'],
                }

            elif self.config.schema == "seacrowd_qa":
                yield data['id'], {
                    "question_id": data['id'],
                    "context": data['context'],
                    "question": data['question'],
                    "answer": {"answer_start": data['answers']['answer_start'][0], "text": data['answers']['text'][0]},
                    "id": data['id'],
                    "choices": [],
                    "type": "",
                    "document_id": data['id'],
                    "meta": {},
                }



"""
Text Pairs Schema
"""
import datasets


def features(label_names=["Yes", "No"]):
    return datasets.Features(
        {
            "id": datasets.Value("string"),
            "text_1": datasets.Value("string"),
            "text_2": datasets.Value("string"),
            "label": datasets.ClassLabel(names=label_names),
        }
    )


def features_with_continuous_label():
    return datasets.Features(
        {
            "id": datasets.Value("string"),
            "text_1": datasets.Value("string"),
            "text_2": datasets.Value("string"),
            "label": datasets.Value("float64"),
        }
    )



import pandas as pd
import urllib.request

def load_x_fact_dataset(dataset_path):
    columns = []
    data = []
    for i, line in enumerate(urllib.request.urlopen(dataset_path)):
        arr = line.decode('utf-8').strip().split('\t')
        if(i == 0):
            columns = arr
        else:
            data.append(arr)
    
    return_dataset = pd.DataFrame(data = data, columns = columns).reset_index()

    return return_dataset[return_dataset['language'] == 'id']



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """\
@inproceedings{mazumder2021mswc,
    author = {Mazumder, Mark and Chitlangia, Sharad and Banbury, Colby and Kang, Yiping and Ciro, Juan and Achorn, Keith and Galvez,
    Daniel and Sabini, Mark and Mattson, Peter and Kanter, David and Diamos, Greg and Warden, Pete and Meyer, Josh and Janapa Reddi,
    Vijay},
    booktitle = {Proceedings of the Neural Information Processing Systems Track on Datasets and Benchmarks},
    editor = {J. Vanschoren and S. Yeung},
    pages = {},
    publisher = {Curran},
    title = {Multilingual Spoken Words Corpus},
    url = {https://datasets-benchmarks-proceedings.neurips.cc/paper_files/paper/2021/file/fe131d7f5a6b38b23cc967316c13dae2-Paper-round2.pdf},
    volume = {1},
    year = {2021}
}
"""

_DATASETNAME = "mswc"

_DESCRIPTION = """\
Multilingual Spoken Words Corpus is a large and growing audio dataset of spoken words in 50 languages collectively spoken by over 5 billion people, for academic research and commercial applications in keyword spotting and spoken term search.
"""

_HOMEPAGE = "https://huggingface.co/datasets/MLCommons/ml_spoken_words"

_LANGUAGES = ["cnh", "ind", "vie"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LANGUAGE_NAME_MAP = {
    "cnh": "cnh",
    "ind": "id",
    "vie": "vi",
}

_FORMATS = ["wav", "opus"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = "https://huggingface.co/datasets/MLCommons/ml_spoken_words/resolve/refs%2Fconvert%2Fparquet/{lang}_{format}/{split}/0000.parquet?download=true"

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]
_SUPPORTED_SCHEMA_STRINGS = [f"seacrowd_{str(TASK_TO_SCHEMA[task]).lower()}" for task in _SUPPORTED_TASKS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class MSWC(datasets.GeneratorBasedBuilder):
    """
    Multilingual Spoken Words Corpus is a large and growing audio dataset of spoken words in 50 languages collectively spoken by over 5 billion people, for academic research and commercial applications in keyword spotting and spoken term search.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = []

    for language in _LANGUAGES:
        for format in _FORMATS:
            subset_id = f"{_DATASETNAME}_{language}_{format}"
            BUILDER_CONFIGS.append(
                SEACrowdConfig(name=f"{subset_id}_source", version=SOURCE_VERSION, description=f"{_DATASETNAME} source schema", schema="source", subset_id=subset_id),
            )

    seacrowd_schema_config: list[SEACrowdConfig] = []

    for seacrowd_schema in _SUPPORTED_SCHEMA_STRINGS:
        for language in _LANGUAGES:
            for format in _FORMATS:
                subset_id = f"{_DATASETNAME}_{language}_{format}"
                seacrowd_schema_config.append(
                    SEACrowdConfig(
                        name=f"{subset_id}_{seacrowd_schema}",
                        version=SEACROWD_VERSION,
                        description=f"{_DATASETNAME} {seacrowd_schema} schema",
                        schema=f"{seacrowd_schema}",
                        subset_id=subset_id,
                    )
                )

    BUILDER_CONFIGS.extend(seacrowd_schema_config)

    DEFAULT_CONFIG_NAME = f"{_LANGUAGES[0]}_{_FORMATS[0]}_source"

    def _info(self) -> datasets.DatasetInfo:

        _, _, format = str(self.config.subset_id).split("_")

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "file": datasets.Value("string"),
                    "is_valid": datasets.Value("bool"),
                    "language": datasets.ClassLabel(num_classes=3),
                    "speaker_id": datasets.Value("string"),
                    "gender": datasets.ClassLabel(num_classes=4),
                    "keyword": datasets.Value("string"),
                    "audio": datasets.Audio(decode=False, sampling_rate=16000 if format == "wav" else 48000),
                }
            )

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.SPEECH_RECOGNITION]).lower()}":
            features = schemas.speech_text_features

        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        split_names = ["train", "validation", "test"]

        result = []

        _, language, format = str(self.config.subset_id).split("_")

        for split_name in split_names:
            path = dl_manager.download_and_extract(_URLS.format(split=split_name, lang=_LANGUAGE_NAME_MAP[language], format=format))

            result.append(
                datasets.SplitGenerator(
                    name=split_name,
                    gen_kwargs={
                        "path": path,
                        "split": split_name,
                        "language": language,
                        "format": format,
                    },
                ),
            )

        return result

    def _generate_examples(self, path: Path, split: str, language: str, format: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        idx = 0

        if self.config.schema == "source":
            df = pd.read_parquet(path)

            for _, row in df.iterrows():
                yield idx, row.to_dict()
                idx += 1

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.SPEECH_RECOGNITION]).lower()}":
            df = pd.read_parquet(path)

            base_folder = os.path.dirname(path)
            base_folder = os.path.join(base_folder, _DATASETNAME, language, format, split)

            if not os.path.exists(base_folder):
                os.makedirs(base_folder)

            audio_paths = []

            for _, row in df.iterrows():
                audio_dict = row["audio"]
                file_name = audio_dict["path"]

                path = os.path.join(base_folder, file_name)

                audio_dict["path"] = path

                with open(path, "wb") as f:
                    f.write(audio_dict["bytes"])

                audio_paths.append(path)

            df.rename(columns={"label": "text"}, inplace=True)

            df["path"] = audio_paths

            df["id"] = df.index + idx
            df = df.assign(text="").astype({"text": "str"})
            df = df.assign(metadata=[{"speaker_age": 0, "speaker_gender": gender} for gender in df["gender"]]).astype({"metadata": "object"})

            df.drop(columns=["file", "is_valid", "language", "gender", "keyword"], inplace=True)

            for _, row in df.iterrows():
                yield idx, row.to_dict()
                idx += 1

        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@article{10.1145/3464378,
author = {Kaing, Hour and Ding, Chenchen and Utiyama, Masao and Sumita, Eiichiro and Sam, Sethserey and Seng, Sopheap and Sudoh, Katsuhito and Nakamura, Satoshi},
title = {Towards Tokenization and Part-of-Speech Tagging for Khmer: Data and Discussion},
year = {2021},
issue_date = {November 2021},
publisher = {Association for Computing Machinery},
address = {New York, NY, USA},
volume = {20},
number = {6},
issn = {2375-4699},
url = {https://doi.org/10.1145/3464378},
doi = {10.1145/3464378},
abstract = {As a highly analytic language, Khmer has considerable ambiguities in tokenization and part-of-speech (POS) tagging processing. This topic is investigated in this study. Specifically, a 20,000-sentence Khmer corpus with manual tokenization and POS-tagging annotation is released after a series of work over the last 4 years. This is the largest morphologically annotated Khmer dataset as of 2020, when this article was prepared. Based on the annotated data, experiments were conducted to establish a comprehensive benchmark on the automatic processing of tokenization and POS-tagging for Khmer. Specifically, a support vector machine, a conditional random field (CRF), a long short-term memory (LSTM)-based recurrent neural network, and an integrated LSTM-CRF model have been investigated and discussed. As a primary conclusion, processing at morpheme-level is satisfactory for the provided data. However, it is intrinsically difficult to identify further grammatical constituents of compounds or phrases because of the complex analytic features of the language. Syntactic annotation and automatic parsing for Khmer will be scheduled in the near future.},
journal = {ACM Trans. Asian Low-Resour. Lang. Inf. Process.},
month = {sep},
articleno = {104},
numpages = {16},
keywords = {annotated data, tokenization, POS-tagging, Khmer, machine learning}
}
"""

_LANGUAGES = ["khm"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "khmer_alt_pos"

_DESCRIPTION = """\
The data contains 20,000-sentence Khmer corpus with manual tokenization and POS-tagging annotation.
"""

_HOMEPAGE = "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/km-nova-181101/README.txt"

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_URL = "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/km-nova-181101.zip"

_SUPPORTED_TASKS = [Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.1.0"

_SEACROWD_VERSION = "1.0.0"


class KhmerAltPOS(datasets.GeneratorBasedBuilder):
    """The data contains 20,000-sentence Khmer corpus with manual tokenization and POS-tagging annotation."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    POS_TAGS = ['n', 'v-', 'v', 'o', 'n[n', 'n]n', 'v]n', '1]n', '.', 'a[n', '1]a', 'a', 'a[v', 'n]a', 'v[a', 'v]v', 'o[o', 'o]o', 'o-', 'a[1', 'v[v', 'a[o', '1', 'a]a', 'n-[n', 'n-]n-', 'a-', 'n]v', 'a]n', 'o]a', 'v]o', 'n-]o', 'a[a', 'n-', 'n[1', 'n]o', '1[1', '1]1', 'n[o', 'n[v', 'a]o', '.]a', 'o-[o-', 'o]o-', 'o[v', 'n[a', 'v-]n', 'v]a', 'o[a', 'o[n', 'o[n-', 'o-]o', 'a-]n', 'n[.', 'o]v', 'o]n', '.]o', 'a-]a', 'a-]o', 'v-[v-', 'o]v-', 'o-]o-', 'v-]o', 'a]v', 'v[n', '.]n', 'n-]a', 'v]o-', 'o[o-', 'n-]n', '+', 'v[o', '.]1', 'v[1', 'n]1', '.]v', 'o]1', 'o[1', '1]o', 'v]1', 'n-[n-', '1]v', '1[n', 'a[.', '.[n', '.].', '+[n', 'n]+', '1[.', '+]o', 'n]o-', 'o-]a', 'v-[v', '.]v-', '.[1', 'v-[o']

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd Seq Label schema",
            schema="seacrowd_seq_label",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features({
                "id": datasets.Value("string"),
                "tokens": [datasets.Value("string")],
                "labels": [datasets.Value("string")],
            })
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.POS_TAGS)

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_path = dl_manager.download_and_extract(_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_path,
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        data_labels = open(f"{filepath}/km-nova-181101/data_km.km-tag.nova", "r").readlines()

        data_tokens = open(f"{filepath}/km-nova-181101/data_km.km-tok.nova", "r").readlines()

        mapping_sentences = {}

        for line in data_tokens:
            id, tokens = line.split("\t")
            tokens, _ = tokens.split("\n")

            if id not in mapping_sentences:
                mapping_sentences[id] = {}
            
            mapping_sentences[id]["tokens"] = tokens.split(" ")

        for line in data_labels:
            id, labels = line.split("\t")
            labels, _ = labels.split("\n")
            
            if id not in mapping_sentences:
                mapping_sentences[id] = {}
            
            mapping_sentences[id]["labels"] = labels.split(" ")

        if self.config.schema == "source" or self.config.schema == "seacrowd_seq_label":
            for num, key in enumerate(mapping_sentences):
                yield num, {
                    "id": key,
                    "tokens": mapping_sentences[key]["tokens"],
                    "labels": mapping_sentences[key]["labels"]
                }

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")



# Taken from https://github.com/valentinakania/indocoref/blob/main/src/utils/feature_utils.py
import re
import nltk

PRONOUNS = ['dia', 'ia', 'beliau', 'mereka', 'kami', 'kita', 'aku', 'saya', 'kamu', 'anda', 'kalian']
PRONOUN_SINGULAR = ['dia', 'ia', 'beliau', 'aku', 'saya', 'kamu', 'anda']
PRONOUN_PLURAL = ['mereka', 'kami', 'kita', 'kalian']
CLITICS = ['mu', 'ku', 'nya']
APPOSITIVES = [',', ';', '(', ')']
COPULATIVES = ['adalah', 'yaitu', 'merupakan', 'ialah', 'yakni']
DEMONSTRATIVES = ['ini', 'itu', 'tersebut']
PERSON = ['orang', 'manusia', 'pria', 'wanita', 'ibu', 'bapak', 'putra', 'putri']
LOCATION = ['tempat', 'lokasi', 'kota', 'provinsi', 'area', 'daerah', 'negara', 'negeri', 'sekitar']
STOPWORDS = [*COPULATIVES, *DEMONSTRATIVES, 'yang', 'itu', 'dan', 'atau', 'tapi', 'nan', 'namun', 'tetapi', 'sang', 'si']

NOUN_POS_TAGS = ["NOUN", "PROPN"]


class FeatureUtils:
    @staticmethod
    def get_chunk_parser():
        grammar = r'''
        NP: {<NUM|DET>* <NOUN|PROPN|PRON>* <.*>* <NOUN|PROPN|PRON>}
            {<NOUN|PROPN|PRON> <NOUN|PROPN|PRON>}
            {<NOUN|PROPN|PRON>* <ADJ>}
            {<NUM|DET>* <NOUN|PROPN|PRON>* <PP>}
            {<NOUN|PROPN|PRON>}
        PP: {<ADP><NOUN|PROPN|PRON>*}
        '''

        chunk_parser = nltk.RegexpParser(grammar)
        return chunk_parser

    @staticmethod
    def find_in_sentence(a, sentences):
        if a.get('sent') is not None and a.get('sent') >= 0:
            return a.get('sent')
        for idx, sent in enumerate(sentences):
            has_all_labels = True
            for l in a['labels']:
                if 'M{}'.format(l) not in sent:
                    has_all_labels = False
                    break
            if has_all_labels:
                return idx
        return 0

    @staticmethod
    def is_pronoun(a):
        return 1 if a.get('text', '').lower() in PRONOUNS or a.get('text', '').lower() in CLITICS else 0

    @staticmethod
    def is_pronoun_singular(a):
        return 1 if a.get('text', '').lower() in PRONOUN_SINGULAR or a.get('text', '').lower() in CLITICS else 0

    @staticmethod
    def is_pronoun_plural(a):
        return 1 if a.get('text', '').lower() in PRONOUN_PLURAL else 0

    @staticmethod
    def is_proper_noun(a):
        return 1 if a.get('pos', '') == 'PROPN' else 0

    @staticmethod
    def find_demonstrative_phrase(a):
        for d in a.get('text').lower().split(' '):
            if d in DEMONSTRATIVES:
                return d
        return None

    @staticmethod
    def strip_demonstrative(a, demonstrative):
        return ' '.join([text for text in a['text'].split(' ') if demonstrative not in text])

    @staticmethod
    def is_ner(a):
        return a.get('ner', 0)

    @staticmethod
    def is_person(a):
        if any((c.lower() in PERSON) for c in a['text']):
            return 1
        return a.get('per', 0)

    @staticmethod
    def is_organization(a):
        return a.get('org', 0)

    @staticmethod
    def is_location(a):
        if any((c.lower() in LOCATION) for c in a['text']):
            return 1
        return a.get('loc', 0)

    @staticmethod
    def is_clitic(a):
        return 1 if a.get('text') in CLITICS else 0

    @staticmethod
    def has_class(a):
        return FeatureUtils.is_person(a) or FeatureUtils.is_organization(a) or FeatureUtils.is_location(a)

    @staticmethod
    def get_head(a):
        tags = a.get('tag', [])
        for t in tags:
            strs, pos = t
            if pos == 'NOUN' or pos == 'PROPN':
                return strs
        return ''

    @staticmethod
    def get_full_head_noun(a):
        head_words = []
        tags = a.get('tag', [])
        for str, pos in tags:
            if pos not in NOUN_POS_TAGS:
                break
            head_words.append(str)
        return ' '.join(head_words)

    @staticmethod
    def get_full_head_proper_noun(a):
        strs = []
        tags = a.get('tag', [])
        for str, pos in tags:
            if pos == 'NOUN' and len(strs) == 0:
                return ''
            if pos == 'PROPN' and str[0].isupper():
                strs.append(str)
        return ' '.join(strs)



class PairFeatureUtils:

    @staticmethod
    def is_same_word_class(a, b):
        return FeatureUtils.is_person(a) and FeatureUtils.is_person(b) or FeatureUtils.is_organization(a) and \
            FeatureUtils.is_organization(b) or FeatureUtils.is_location(a) and FeatureUtils.is_location(b)

    @staticmethod
    def is_word_class_mismatch(a, b):
        return FeatureUtils.has_class(a) and FeatureUtils.has_class(b) and not PairFeatureUtils.is_same_word_class(a, b)
    
    @staticmethod
    def is_exact_match(a, b):
        textA = re.sub('[^0-9a-zA-Z -,.]+', '', a.get('text'))
        textB = re.sub('[^0-9a-zA-Z -,.]+', '', b.get('text'))
        return 1 if textA == textB else 0

    @staticmethod
    def is_name_shortened(a, b):
        textA = re.sub('[^0-9a-zA-Z -,.]+', '', a.get('text').lower())
        textB = re.sub('[^0-9a-zA-Z -,.]+', '', b.get('text').lower())
        if textA == '' or textB == '':
            return 0
        return 1 if (textA in textB or textB in textA) \
            and FeatureUtils.is_ner(a) and FeatureUtils.is_ner(b) and PairFeatureUtils.is_same_word_class(a, b) else 0

    @staticmethod
    def is_appositive(a, b, sentences):
        idxA = FeatureUtils.find_in_sentence(a, sentences)
        idxB = FeatureUtils.find_in_sentence(b, sentences)
        if idxA != idxB:
            return 0
        if idxA >= len(sentences):
            token = len(' '.join(sentences).split('.'))
            idxA -= (token - len(sentences))
        try:
            sent = sentences[idxA]
            start = min(sent.find(a.get('text')), sent.find(b.get('text'))) + len(a.get('text'))
            end = max(sent.find(a.get('text')), sent.find(b.get('text')))
        except:
            return 0
        if sent[start:end].count(' ') > 1:
            return 0
        if any((c.lower() in APPOSITIVES) for c in sent[start:end]):
            return FeatureUtils.is_ner(a) ^ FeatureUtils.is_ner(b)
        return 0

    @staticmethod
    def is_copulative(a, b, sentences):
        idxA = FeatureUtils.find_in_sentence(a, sentences)
        idxB = FeatureUtils.find_in_sentence(b, sentences)
        if idxA != idxB:
            return 0
        if idxA >= len(sentences):
            token = len(' '.join(sentences).split('.'))
            idxA -= (token - len(sentences))
        try:
            sent = sentences[idxA]
            start = min(sent.find(a.get('text')), sent.find(b.get('text'))) + len(a.get('text'))
            end = max(sent.find(a.get('text')), sent.find(b.get('text')))
        except:
            return 0
        return 1 if any((c.lower() in COPULATIVES) for c in sent[start:end].split()) else 0

    @staticmethod
    def is_demonstrative(a, b):
        d = FeatureUtils.find_demonstrative_phrase(b)
        if not d:
            return 0
        cleaned_b = FeatureUtils.strip_demonstrative(b, d)
        for c in cleaned_b.lower().split(' '):
            if c in STOPWORDS:
                continue
            if c not in a['text'].lower().split(' '):
                return 0
        return 1

    @staticmethod
    def is_abbreviation(a, b):
        if a['text'].lower() == b['text'].lower():
            return 0
        short = a.get('text') if len(a) < len(b) else b.get('text')
        long = a.get('text') if len(a) >= len(b) else b.get('text')
        short = short.replace('.', '')
        longs = long.split(' ')
        abbreviation = ''.join([c[0] for c in longs if len(c) > 1])
        if abbreviation.lower() == short.lower():
            return 1
        return 0


    @staticmethod
    def is_relaxed_match(a, b):
        if a['text'].lower() == b['text'].lower():
            return 0
        short = a if len(a['text']) < len(b['text']) else b
        long = a if len(a['text']) >= len(b['text']) else b
        short_full_head = FeatureUtils.get_full_head_noun(short).lower()
        long_full_head = FeatureUtils.get_full_head_noun(long).lower()
        for c in short_full_head.split(' '):
            if c not in long_full_head.split(' '):
                return 0
        return 1 if PairFeatureUtils.is_same_word_class(a, b) else 0
        
    @staticmethod
    def is_head_match(a, b):
        if FeatureUtils.get_head(a).lower() == FeatureUtils.get_head(b).lower():
            return 1
        return 0

    @staticmethod
    def is_full_proper_head_match(a, b):
        a_propn_head = FeatureUtils.get_full_head_proper_noun(a).lower()
        b_propn_head = FeatureUtils.get_full_head_proper_noun(b).lower()
        if a_propn_head == '' or b_propn_head == '':
            return 0
        short_head = a_propn_head if len(a_propn_head) < len(b_propn_head) else b_propn_head
        long_head = a_propn_head if len(a_propn_head) >= len(b_propn_head) else b_propn_head
        for word in short_head.split(' '):
            if word not in long_head.split(' '):
                return 0
        return 1


# coding=utf-8
import json

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_DATASETNAME = "iapp_squad"
_CITATION = """\
@dataset
{
  kobkrit_viriyayudhakorn_2021_4539916,
  author       = {Kobkrit Viriyayudhakorn and Charin Polpanumas},
  title        = {iapp_wiki_qa_squad},
  month        = feb,
  year         = 2021,
  publisher    = {Zenodo},
  version      = 1,
  doi          = {10.5281/zenodo.4539916},
  url          = {https://doi.org/10.5281/zenodo.4539916}
}
"""

_DESCRIPTION = """
`iapp_wiki_qa_squad` is an extractive question answering dataset from Thai Wikipedia articles.
It is adapted from [the original iapp-wiki-qa-dataset](https://github.com/iapp-technology/iapp-wiki-qa-dataset)
to [SQuAD](https://rajpurkar.github.io/SQuAD-explorer/) format, resulting in
5761/742/739 questions from 1529/191/192 articles.
"""

_HOMEPAGE = "https://github.com/iapp-technology/iapp-wiki-qa-dataset"
_LICENSE = Licenses.MIT.value
_HF_URL = " https://huggingface.co/datasets/iapp_wiki_qa_squad"
_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]
_LOCAL = False
_LANGUAGES = ["tha"]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_URLS = {
    "train": "https://raw.githubusercontent.com/iapp-technology/iapp-wiki-qa-dataset/main/squad_format/data/train.jsonl",
    "validation": "https://raw.githubusercontent.com/iapp-technology/iapp-wiki-qa-dataset/main/squad_format/data/valid.jsonl",
    "test": "https://raw.githubusercontent.com/iapp-technology/iapp-wiki-qa-dataset/main/squad_format/data/test.jsonl",
}


class IappWikiQASquadDataset(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        SEACrowdConfig(name=f"{_DATASETNAME}_source", version=datasets.Version(_SOURCE_VERSION), description=_DESCRIPTION, subset_id=f"{_DATASETNAME}", schema="source"),
        SEACrowdConfig(name=f"{_DATASETNAME}_seacrowd_qa", version=datasets.Version(_SEACROWD_VERSION), description=_DESCRIPTION, subset_id=f"{_DATASETNAME}", schema="seacrowd_qa"),
    ]
    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "question_id": datasets.Value("string"),
                    "article_id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.features.Sequence(
                        {
                            "text": datasets.Value("string"),
                            "answer_start": datasets.Value("int32"),
                            "answer_end": datasets.Value("int32"),
                        }
                    ),
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features["meta"] = {
                "answer_start": datasets.Value("int32"),
                "answer_end": datasets.Value("int32"),
            }
        return datasets.DatasetInfo(description=_DESCRIPTION, features=features, homepage=_HOMEPAGE, citation=_CITATION, license=_LICENSE)

    def _split_generators(self, dl_manager):
        file_paths = dl_manager.download_and_extract(_URLS)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": file_paths["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": file_paths["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": file_paths["test"]},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                if self.config.schema == "source":
                    yield id_, {
                        "question_id": data["question_id"],
                        "article_id": data["article_id"],
                        "title": data["title"],
                        "context": data["context"],
                        "question": data["question"],
                        "answers": {
                            "text": data["answers"]["text"],
                            "answer_start": data["answers"]["answer_start"],
                            "answer_end": data["answers"]["answer_end"],
                        },
                    }
                elif self.config.schema == "seacrowd_qa":
                    yield id_, {
                        "id": id_,
                        "question_id": data["question_id"],
                        "document_id": data["article_id"],
                        "question": data["question"],
                        "type": "abstractive",
                        "choices": [],
                        "context": data["context"],
                        "answer": data["answers"]["text"],
                        "meta": {"answer_start": data["answers"]["answer_start"][0], "answer_end": data["answers"]["answer_end"][0]},
                    }



import os
from typing import Dict, List, Tuple

import datasets
import jsonlines as jl
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{thapliyal-etal-2022-crossmodal,
    title = "Crossmodal-3600: A Massively Multilingual Multimodal Evaluation Dataset",
    author = "Thapliyal, Ashish V.  and
      Pont Tuset, Jordi  and
      Chen, Xi  and
      Soricut, Radu",
    editor = "Goldberg, Yoav  and
      Kozareva, Zornitsa  and
      Zhang, Yue",
    booktitle = "Proceedings of the 2022 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2022",
    address = "Abu Dhabi, United Arab Emirates",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.emnlp-main.45",
    doi = "10.18653/v1/2022.emnlp-main.45",
    pages = "715--729",
}
"""

_DATASETNAME = "xm3600"

_DESCRIPTION = """\
Crossmodal-3600 dataset (XM3600 in short), a geographically-diverse set of 3600 images annotated with
human-generated reference captions in 36 languages. The images were selected from across the world,
covering regions where the languages are spoken, and annotated with captions that achieve consistency in
terms of style across all languages, while avoiding annotation artifacts due to direct translation.
The languages covered in the dataset include Filipino, Indonesian, Thai, and Vietnamnese
"""

_HOMEPAGE = "https://google.github.io/crossmodal-3600/"

_LICENSE = Licenses.CC_BY_4_0.value

_URLS = {
    "captions": "https://google.github.io/crossmodal-3600/web-data/captions.zip",
    "images": "https://open-images-dataset.s3.amazonaws.com/crossmodal-3600/images.tgz",
    "image_attributions": "https://google.github.io/crossmodal-3600/web-data/image_attributions.csv",
}

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = ["fil", "id", "th", "vi"]

_LOCAL = False


class XM3600Dataset(datasets.GeneratorBasedBuilder):
    """
    Crossmodal-3600 dataset (XM3600 in short), a geographically-diverse set of 3600 images annotated with
    human-generated reference captions in 36 languages. The images were selected from across the world,
    covering regions where the languages are spoken, and annotated with captions that achieve consistency in
    terms of style across all languages, while avoiding annotation artifacts due to direct translation.
    The languages covered in the dataset include Filipino, Indonesian, Thai, and Vietnamnese
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME}_{lang} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_{lang}",
        )
        for lang in _LANGUAGES
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_seacrowd_imtext",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME}_{lang} SEACrowd schema",
            schema="seacrowd_imtext",
            subset_id=f"{_DATASETNAME}_{lang}",
        )
        for lang in _LANGUAGES
    ]

    DEFAULT_CONFIG_NAME = f"xm3600_{sorted(_LANGUAGES)[0]}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "image_paths": datasets.Value("string"),
                    "texts": {
                        "caption": datasets.Value("string"),
                        "caption/tokenized": datasets.Value("string"),
                        "caption/tokenized/lowercase": datasets.Value("string"),
                    },
                }
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        captions_path = dl_manager.download_and_extract(_URLS["captions"])
        images_path = dl_manager.download_and_extract(_URLS["images"])
        attr_path = dl_manager.download(_URLS["image_attributions"])

        train_caps = {}
        test_caps = {}
        val_caps = {}

        current_lang = self.config.subset_id.split("_")[1]

        img_df = pd.read_csv(attr_path)

        img_df_train = img_df.loc[img_df["Subset"] == "train"][["ImageID", "Subset"]]
        img_df_test = img_df.loc[img_df["Subset"] == "test"][["ImageID", "Subset"]]
        img_df_val = img_df.loc[img_df["Subset"] == "validation"][["ImageID", "Subset"]]

        with jl.open(os.path.join(captions_path, "captions.jsonl"), mode="r") as jsonl_file:
            for line in jsonl_file:
                if line["image/key"] in img_df_train.ImageID.values:
                    train_caps[line["image/key"]] = line[current_lang]
                elif line["image/key"] in img_df_test.ImageID.values:
                    test_caps[line["image/key"]] = line[current_lang]
                elif line["image/key"] in img_df_val.ImageID.values:
                    val_caps[line["image/key"]] = line[current_lang]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {"img_ids": img_df_train.ImageID.values, "images": {img_id: os.path.join(images_path, img_id + ".jpg") for img_id in img_df_train.ImageID.values}, "captions": train_caps},
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {"img_ids": img_df_test.ImageID.values, "images": {img_id: os.path.join(images_path, img_id + ".jpg") for img_id in img_df_test.ImageID.values}, "captions": test_caps},
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": {"img_ids": img_df_val.ImageID.values, "images": {img_id: os.path.join(images_path, img_id + ".jpg") for img_id in img_df_val.ImageID.values}, "captions": val_caps},
                },
            ),
        ]

    def _generate_examples(self, filepath: dict) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        counter = 0
        for img_id in filepath["img_ids"]:
            cap = filepath["captions"][img_id]
            for line in cap["caption"]:
                cap_index = cap["caption"].index(line)
                if self.config.schema == "source":
                    yield counter, {
                        "id": img_id + "_" + str(counter),
                        "image_paths": filepath["images"][img_id],
                        "texts": {
                            "caption": line,
                            "caption/tokenized": cap["caption/tokenized"][cap_index],
                            "caption/tokenized/lowercase": cap["caption/tokenized/lowercase"][cap_index],
                        },
                    }

                elif self.config.schema == "seacrowd_imtext":
                    yield counter, {
                        "id": img_id + "_" + str(counter),
                        "image_paths": [filepath["images"][img_id]],
                        "texts": line,
                        "metadata": {
                            "context": None,
                            "labels": None,
                        },
                    }

                else:
                    raise ValueError(f"Invalid config: {self.config.name}")

                counter += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{elkishky_ccaligned_2020,
    author = {El-Kishky, Ahmed and Chaudhary, Vishrav and Guzm{\'a}n, Francisco and Koehn, Philipp},
    booktitle = {Proceedings of the 2020 Conference on Empirical Methods in Natural Language Processing (EMNLP 2020)},
    month = {November},
    title = {{CCAligned}: A Massive Collection of Cross-lingual Web-Document Pairs},
    year = {2020}
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.emnlp-main.480",
    doi = "10.18653/v1/2020.emnlp-main.480",
    pages = "5960--5969"
}
"""

_DATASETNAME = "cc_aligned_doc"

_DESCRIPTION = """\
CCAligned consists of parallel or comparable web-document pairs in 137 languages aligned with English\
(10 languages are from Southeast Asia; Burmese has two document collection with different scripts).\
These web-document pairs were constructed by performing language identification on raw web-documents, \
and ensuring corresponding language codes were corresponding in the URLs of web documents. This pattern \
matching approach yielded more than 100 million aligned documents paired with English.
"""

_HOMEPAGE = "https://www2.statmt.org/cc-aligned/"

_LANGUAGES = ["ind", "sun", "tha", "vie", "zlm", "lao", "khm", "mya", "ceb", "war"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False
_SUBSETS = {"id_ID": "ind", "su_ID": "sun", "th_TH": "tha", "vi_VN": "vie", "ms_MY": "zlm", "lo_LA": "lao", "km_KH": "khm", "my_MM": "mya", "my_MM_zaw": "mya", "cx_PH": "ceb", "wy_PH": "war"}
_URLS = {_DATASETNAME: "https://data.statmt.org/cc-aligned/en_XX-{subset}.tsv.xz"}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class CCAlignedDocDataset(datasets.GeneratorBasedBuilder):

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "t2t"

    BUILDER_CONFIGS = [SEACrowdConfig(name=f"{_DATASETNAME}_{subset}_source", version=datasets.Version(_SOURCE_VERSION), description=f"{_DATASETNAME} source schema", schema="source", subset_id=f"{_DATASETNAME}",) for subset in _SUBSETS.keys()] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_seacrowd_{schema_name}",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{schema_name}",
            subset_id=f"{_DATASETNAME}",
        )
        for subset, schema_name in zip(_SUBSETS.keys(), len(_SUBSETS.keys()) * [SEACROWD_SCHEMA_NAME])
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_id_ID_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "Domain": datasets.Value("string"),
                    "Source_URL": datasets.Value("string"),
                    "Source_Content": datasets.Value("string"),
                    "Target_URL": datasets.Value("string"),
                    "Target_Content": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        subset = "_".join([self.config.name.split("_")[3], self.config.name.split("_")[4]])
        urls = _URLS[_DATASETNAME].format(subset=subset)
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        subset = "_".join([self.config.name.split("_")[3], self.config.name.split("_")[4]])
        lines = open(filepath, "r").readlines()
        if self.config.schema == "source":
            idx = 0
            for line in lines:
                content = line.split("\t")
                example = {
                    "Domain": content[0],
                    "Source_URL": content[1],
                    "Source_Content": content[2],
                    "Target_URL": content[3],
                    "Target_Content": content[4],
                }
                yield idx, example
                idx += 1
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            idx = 0
            for line in lines:
                content = line.split("\t")
                example = {
                    "id": str(idx),
                    "text_1": content[2],
                    "text_2": content[4],
                    "text_1_name": "en",
                    "text_2_name": _SUBSETS[subset],
                }
                yield idx, example
                idx += 1



"""
This new update refers to the this HF dataloader script
https://huggingface.co/datasets/csebuetnlp/xlsum/blob/main/xlsum.py
while conforming to SEACrowd schema
"""

import os
from pathlib import Path
from typing import Dict, List, Tuple

import json
import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks


_CITATION = """\
@inproceedings{hasan2021xl,
  title={XL-Sum: Large-Scale Multilingual Abstractive Summarization for 44 Languages},
  author={Hasan, Tahmid and Bhattacharjee, Abhik and Islam, Md Saiful and Mubasshir, Kazi and Li, Yuan-Fang and Kang, Yong-Bin and Rahman, M Sohel and Shahriyar, Rifat},
  booktitle={Findings of the Association for Computational Linguistics: ACL-IJCNLP 2021},
  pages={4693--4703},
  year={2021}
}
"""

_LOCAL = False
_LANGUAGES = ["ind", "mya", "tha", "vie", "eng"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LANG_TO_DATASOURCE_LANG = {
    "ind": "indonesian",
    "mya": "burmese",
    "vie": "vietnamese",
    "tha": "thai"}

_DATASETNAME = "xl_sum"

_DESCRIPTION = """\
XL-Sum, a comprehensive and diverse dataset comprising 1 million professionally annotated article-summary pairs from BBC, was extracted using a set of carefully designed heuristics.
The dataset covers 44 languages ranging from low to high-resource, including 4 indigenous languages spoken in Southeast Asia region.
"""

_HOMEPAGE = "https://github.com/csebuetnlp/xl-sum"

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_URLS = "https://huggingface.co/datasets/csebuetnlp/xlsum/resolve/main/data/{}_XLSum_v{}.tar.bz2"

_SUPPORTED_TASKS = [Tasks.SUMMARIZATION]

_SOURCE_VERSION = "2.0.0"

_SEACROWD_VERSION = "1.0.0"


def construct_configs_on_langs() -> List[SEACrowdConfig]:
    """
    The function `construct_configs` constructs a list of SEACrowdConfig objects based on `_LANGUAGES` var, and returns the list.

    output:
        a list of `SEACrowdConfig` objects based on instantiated init variables
    """

    # set output var
    config_list = []

    # construct zipped arg for config instantiation
    CONFIG_SUFFIXES_FOR_TASK = [TASK_TO_SCHEMA.get(task).lower() for task in _SUPPORTED_TASKS]
    TASKS_AND_CONFIG_SUFFIX_PAIRS = list(zip(_SUPPORTED_TASKS, CONFIG_SUFFIXES_FOR_TASK))

    # implement source schema
    version, config_name_prefix = _SOURCE_VERSION, "source"
    config_list += [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}",
            version=datasets.Version(version),
            description=f"{_DATASETNAME} {config_name_prefix} schema for language code {_LANG}",
            schema=f"{config_name_prefix}",
            subset_id=_LANG,
        )
        #skip english lang
        for _LANG in _LANGUAGES if _LANG != "eng"
    ]

    # implement SEACrowd schema
    version, config_name_prefix = _SEACROWD_VERSION, "seacrowd"
    for task_obj, config_name_suffix in TASKS_AND_CONFIG_SUFFIX_PAIRS:
        config_list += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}_{config_name_suffix}",
                version=datasets.Version(version),
                description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name} and language code {_LANG}",
                schema=f"{config_name_prefix}_{config_name_suffix}",
                subset_id=_LANG,
            )
            #skip english lang
            for _LANG in _LANGUAGES if _LANG != "eng"
        ]
    return config_list


class XLSum(datasets.GeneratorBasedBuilder):
    """XL-Sum is a large-scale multilingual summarization dataset that covers 45 languages including Indonesian text summarization."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = construct_configs_on_langs()

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
               {
                    "id": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "summary": datasets.Value("string")
               }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )


    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        lang = _LANG_TO_DATASOURCE_LANG[self.config.subset_id]
        url = _URLS.format(lang, self.SOURCE_VERSION.version_str[:-2])

        data_dir = dl_manager.download_and_extract(url)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, lang + "_train.jsonl"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, lang + "_test.jsonl"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, lang + "_val.jsonl"),
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:

        if self.config.schema == "source":
            with open(filepath, encoding="utf-8") as f:
                for row in f:
                    data = json.loads(row)
                    ex = {
                        "id": data["id"],
                        "url": data["url"],
                        "title": data["title"],
                        "text": data["text"],
                        "summary": data["summary"],
                    }
                    yield data["id"], ex

        elif self.config.schema == "seacrowd_t2t":
            # the title is unused for this schema
            with open(filepath, encoding="utf-8") as f:
                for row in f:
                    data = json.loads(row)
                    ex = {
                        "id": data["id"],
                        "text_1": data["text"],
                        "text_2": data["summary"],
                        "text_1_name": "text",
                        "text_2_name": "summary"
                    }
                    yield data["id"], ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
The khPOS Corpus (Khmer POS Corpus) is a 12,000 sentences (25,626 words) manually word segmented and POS tagged corpus 
developed for Khmer language NLP research and developments. We collected Khmer sentences from websites that include
various area such as economics, news, politics. Moreover it is also contained some student list and voter list of 
national election committee of Cambodia. The average number of words per sentence in the whole corpus is 10.75. 
Here, some symbols such as "។" (Khmer sign Khan), "៖" (Khmer sign Camnuc pii kuuh), "-", "?", "[", "]" etc. also 
counted as words. The shortest sentence contained only 1 word and longest sentence contained 169 words. This dataset contains
A validation set and a test set, each containing 1000 sentences.
"""
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@inproceedings{kyaw2017comparison,
  title={Comparison of Six POS Tagging Methods on 12K Sentences Khmer Language POS Tagged Corpus},
  author={Ye Kyaw Thu and Vichet Chea and Yoshinori Sagisaka},
  booktitle={Proceedings of the first Regional Conference on Optical character recognition and Natural language processing technologies for ASEAN languages (ONA 2017)},
  year={2017},
  month={December 7-8},
  address={Phnom Penh, Cambodia}
}
"""

_DATASETNAME = "khpos"

_DESCRIPTION = """\
The khPOS Corpus (Khmer POS Corpus) is a 12,000 sentences (25,626 words) manually word segmented and POS tagged corpus 
developed for Khmer language NLP research and developments. We collected Khmer sentences from websites that include
various area such as economics, news, politics. Moreover it is also contained some student list and voter list of 
national election committee of Cambodia. The average number of words per sentence in the whole corpus is 10.75. 
Here, some symbols such as "។" (Khmer sign Khan), "៖" (Khmer sign Camnuc pii kuuh), "-", "?", "[", "]" etc. also 
counted as words. The shortest sentence contained only 1 word and longest sentence contained 169 words. This dataset contains
A validation set and a test set, each containing 1000 sentences.
"""

_HOMEPAGE = "https://github.com/ye-kyaw-thu/khPOS/tree/master"

_LANGUAGES = ['khm']  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value 

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        'train': "https://raw.githubusercontent.com/ye-kyaw-thu/khPOS/master/corpus-draft-ver-1.0/data/after-replace/train.all2",
        'validation': "https://raw.githubusercontent.com/ye-kyaw-thu/khPOS/master/corpus-draft-ver-1.0/data/OPEN-TEST",
        'test': "https://raw.githubusercontent.com/ye-kyaw-thu/khPOS/master/corpus-draft-ver-1.0/data/CLOSE-TEST"
    }
}

_SUPPORTED_TASKS = [Tasks.POS_TAGGING]  

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class KhPOS(datasets.GeneratorBasedBuilder):
    """\
This datasets contain 12000 sentences (25626 words) for the Khmer language.
There are 24 POS tags and their description can be found at https://github.com/ye-kyaw-thu/khPOS/tree/master.
The used Khmer Tokenizer can be found in the above github repository as well. This dataset contains
A validation set and a test set, each containing 1000 sentences.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="khpos_source",
            version=SOURCE_VERSION,
            description="khpos source schema",
            schema="source",
            subset_id="khpos",
        ),
        SEACrowdConfig(
            name="khpos_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description="khpos SEACrowd schema",
            schema="seacrowd_seq_label",
            subset_id="khpos",
        ),
    ]

    DEFAULT_CONFIG_NAME = "khpos_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({
                "id"      : datasets.Value("string"),
                "tokens"  : datasets.Sequence(datasets.Value("string")),
                #pos_tags follows order from corpus-draft-ver-1.0/data/after-replace/train.all2.tag.freq
                "pos_tags": datasets.Sequence(datasets.features.ClassLabel(
                    names = [
                        'AB', 'AUX', 'CC', 'CD',
                        'DBL', 'DT', 'ETC', 'IN',
                        'JJ', 'KAN', 'M', 'NN',
                        'PA', 'PN', 'PRO', 'QT',
                        'RB', 'RPN', 'SYM', 'UH',
                        'VB', 'VB_JJ', 'VCOM'
                    ]
                ))
            })
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label.features([
                        'AB', 'AUX', 'CC', 'CD',
                        'DBL', 'DT', 'ETC', 'IN',
                        'JJ', 'KAN', 'M', 'NN',
                        'PA', 'PN', 'PRO', 'QT',
                        'RB', 'RPN', 'SYM', 'UH',
                        'VB', 'VB_JJ', 'VCOM'
                    ])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]['train']
        path = dl_manager.download_and_extract(urls)

        dev_url  = _URLS[_DATASETNAME]['validation']
        dev_path = dl_manager.download_and_extract(dev_url)

        test_url  = _URLS[_DATASETNAME]['test']
        test_path = dl_manager.download_and_extract(test_url)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": path,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": dev_path,
                    "split": "dev",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        with open(filepath, encoding="utf-8") as file:
            counter = 0
            for line in file:
                if line.strip() != "":
                    groups = line.split(" ")
                    tokens = []
                    pos_tags = []
                    for group in groups:
                        token, pos_tag = group.split("/")
                        tokens.append(token)
                        pos_tags.append(pos_tag)
                    if self.config.schema == "source":
                        yield (
                            counter,
                            {
                                "id"      : str(counter),
                                "tokens"  : tokens,
                                "pos_tags": pos_tags
                            }
                        )
                        counter += 1
                    elif self.config.schema == "seacrowd_seq_label":
                        yield (
                            counter,
                            {
                                "id"    : str(counter),
                                "tokens": tokens,
                                "labels": pos_tags
                            }
                        )
                        counter += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{palen-michel-lignos-2023-lr,
  author    = {Palen-Michel, Chester and Lignos, Constantine},
  title     = {LR - Sum: Summarization for Less-Resourced Languages},
  booktitle = {Findings of the Association for Computational Linguistics: ACL 2023},
  year      = {2023},
  publisher = {Association for Computational Linguistics},
  address   = {Toronto, Canada},
  doi       = {10.18653/v1/2023.findings-acl.427},
  pages     = {6829--6844},
}
"""

_LOCAL = False
_LANGUAGES = ["ind", "khm", "lao", "mya", "tha", "vie"]

_DATASETNAME = "lr_sum"
_DESCRIPTION = """
LR-Sum is a news abstractive summarization dataset focused on low-resource languages. It contains human-written summaries
for 39 languages and the data is based on the Multilingual Open Text corpus
(ultimately derived from the Voice of America website).
"""

_HOMEPAGE = "https://huggingface.co/datasets/bltlab/lr-sum"
_LICENSE = Licenses.CC_BY_4_0.value
_URL = "https://huggingface.co/datasets/bltlab/lr-sum"

_SUPPORTED_TASKS = [Tasks.SUMMARIZATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class LRSumDataset(datasets.GeneratorBasedBuilder):
    """Dataset of article-summary pairs for different low-resource languages."""

    # Config to load individual datasets per language
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema for {lang} language",
            schema="source",
            subset_id=f"{_DATASETNAME}_{lang}",
        )
        for lang in _LANGUAGES
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{lang}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema for {lang} language",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}_{lang}",
        )
        for lang in _LANGUAGES
    ]

    # Config to load all datasets
    BUILDER_CONFIGS.extend(
        [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME} source schema for all languages",
                schema="source",
                subset_id=_DATASETNAME,
            ),
            SEACrowdConfig(
                name=f"{_DATASETNAME}_seacrowd_t2t",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema for all languages",
                schema="seacrowd_t2t",
                subset_id=_DATASETNAME,
            ),
        ]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "summary": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # dl_manager not used since dataloader uses HF 'load_dataset'
        return [
            datasets.SplitGenerator(name=split, gen_kwargs={"split": split._name})
            for split in (
                datasets.Split.TRAIN,
                datasets.Split.VALIDATION,
                datasets.Split.TEST,
            )
        ]

    def _load_hf_data_from_remote(self, lang: str, split: str) -> datasets.DatasetDict:
        """Load dataset from HuggingFace."""
        hf_remote_ref = "/".join(_URL.split("/")[-2:])
        return datasets.load_dataset(hf_remote_ref, lang, split=split)

    def _generate_examples(self, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        lr_sum_datasets = []

        lang = self.config.subset_id.split("_")[-1]
        if lang in _LANGUAGES:
            lr_sum_datasets.append(self._load_hf_data_from_remote(lang, split))
        else:
            for lang in _LANGUAGES:
                lr_sum_datasets.append(self._load_hf_data_from_remote(lang, split))

        index = 0
        for lang_subset in lr_sum_datasets:
            for row in lang_subset:
                if self.config.schema == "source":
                    example = row

                elif self.config.schema == "seacrowd_t2t":
                    example = {
                        "id": str(index),
                        "text_1": row["text"],
                        "text_2": row["summary"],
                        "text_1_name": "document",
                        "text_2_name": "summary",
                    }
                yield index, example
                index += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_ud_data, load_ud_data_as_seacrowd_kb
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@unpublished{Alfina2023,
    author = {Alfina, Ika and Yuliawati, Arlisa and Tanaya, Dipta and Dinakaramani, Arawinda and Zeman, Daniel},
    title = {{A Gold Standard Dataset for Javanese Tokenization, POS Tagging, Morphological Feature Tagging, and Dependency Parsing}},
    year = {2023}
}
"""

_DATASETNAME = "ud_jv_csui"

_DESCRIPTION = """\
UD Javanese-CSUI is a dependency treebank in Javanese, a regional language in Indonesia with more than 68 million users.
It was developed by Alfina et al. from the Faculty of Computer Science, Universitas Indonesia.
The newest version has 1000 sentences and 14K words with manual annotation.

The sentences use the Latin script and do not use the original writing system of Javanese (Hanacaraka).

The original sentences were taken from several resources:
1. Javanese reference grammar books (125 sents)
2. OPUS, especially from the Javanese section of the WikiMatrix v1 corpus (150 sents)
3. Online news (Solopos) (725 sents)

Javanese has several language levels (register), such as Ngoko, Krama, Krama Inggil, and Krama Andhap.
In this treebank, the sentences predominantly use Ngoko words, some of which use Krama words.
"""

_HOMEPAGE = "https://github.com/UniversalDependencies/UD_Javanese-CSUI"

_LANGUAGES = ["jav"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/UniversalDependencies/UD_Javanese-CSUI/master/jv_csui-ud-test.conllu",
}

_SUPPORTED_TASKS = [Tasks.DEPENDENCY_PARSING, Tasks.MACHINE_TRANSLATION, Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


def _resolve_misannotation_(dataset):
    """Resolving mis-annotation in the raw data. In-place."""
    for d in dataset:
        # Metadata's typos
        if d["sent_id"] == "opus-wiki-5":  # From the raw file. Thrown-away during parsing due to no field name.
            d.setdefault("text_en", "Prior to World War II, 14 commercial and 12 public radios could be operated in France.")
        if d["sent_id"] == "wedhawati-2001-66":  # empty string
            d.setdefault("text_en", "Reading can expand knowledge.")
        if d["sent_id"] == "opus-wiki-72":
            d["text_en"] = d.pop("text-en")  # metadata mis-titled
        if d["sent_id"] == "opus-wiki-27":
            d["text_id"] = d.pop("tex_id")  # metadata mis-titled

        # Problems on the annotation itself
        if d["sent_id"] == "solopos-2022-42":  # POS tag is also wrong. Proceed with caution.
            d["form"][1] = d["form"][1].replace("tresnane", "tresna")  # tresna + e
        if d["sent_id"] == "solopos-2022-93":  # wrong annot
            d["form"][10] = d["form"][10].replace("tengene", "tengen")  # tengen + e
        if d["sent_id"] == "solopos-2022-506":  # annotation inconsistency on occurrences of word "sedina"
            d["form"][3] = d["form"][3].replace("siji", "se")
        if d["sent_id"] == "solopos-2022-711":  # annotation inconsistency on the word "rasah" from "ra" and "usah"
            d["form"][11] = d["form"][11].replace("usah", "sah")

    return dataset


class UdJvCsuiDataset(datasets.GeneratorBasedBuilder):
    """Treebank of Javanese comprises 1030 sentences from 14K words with manual annotation"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    # source: https://universaldependencies.org/u/pos/
    UPOS_TAGS = ["ADJ", "ADP", "ADV", "AUX", "CCONJ", "DET", "INTJ", "NOUN", "NUM", "PART", "PRON", "PROPN", "PUNCT", "SCONJ", "SYM", "VERB", "X"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_kb",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd KB schema",
            schema="seacrowd_kb",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd Text-to-Text schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd Seq Label schema",
            schema="seacrowd_seq_label",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    # metadata
                    "sent_id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "text_id": datasets.Value("string"),
                    "text_en": datasets.Value("string"),
                    # tokens
                    "id": [datasets.Value("string")],
                    "form": [datasets.Value("string")],
                    "lemma": [datasets.Value("string")],
                    "upos": [datasets.Value("string")],
                    "xpos": [datasets.Value("string")],
                    "feats": [datasets.Value("string")],
                    "head": [datasets.Value("string")],
                    "deprel": [datasets.Value("string")],
                    "deps": [datasets.Value("string")],
                    "misc": [datasets.Value("string")],
                }
            )
        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.UPOS_TAGS)

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_path = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,  # https://github.com/UniversalDependencies/UD_Javanese-CSUI#split
                gen_kwargs={"filepath": data_path},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        # Note from hudi_f:
        #   Other than 3 sentences with multi-span of length 3, the data format seems fine.
        #   Thus, it is safe to ignore the assertion. (as of 2024/02/14)
        dataset = list(
            load_ud_data(
                filepath,
                filter_kwargs={"id": lambda i: isinstance(i, int)},
                # assert_fn=assert_multispan_range_is_one
            )
        )
        _resolve_misannotation_(dataset)

        for d in dataset:
            if "text_id" not in d or "text_en" not in d:
                print(d)

        if self.config.schema == "source":
            pass

        elif self.config.schema == "seacrowd_kb":
            dataset = load_ud_data_as_seacrowd_kb(
                filepath,
                dataset,
                morph_exceptions=[
                    # Exceptions due to inconsistencies in the raw data annotation
                    ("ne", "e"),
                    ("nipun", "ipun"),
                    ("me", "e"),  # occurrence word: "Esemme" = "Esem" + "e". original text has double 'm'.
                ],
            )

        elif self.config.schema == "seacrowd_t2t":
            dataset = list(
                map(
                    lambda d: {
                        "id": d["sent_id"],
                        "text_1": d["text"],
                        "text_2": d["text_id"],
                        "text_1_name": "jav",
                        "text_2_name": "ind",
                    },
                    dataset,
                )
            )

        elif self.config.schema == "seacrowd_seq_label":
            dataset = list(
                map(
                    lambda d: {
                        "id": d["sent_id"],
                        "tokens": d["form"],
                        "labels": d["upos"],
                    },
                    dataset,
                )
            )

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        for key, example in enumerate(dataset):
            yield key, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_DATASETNAME = "melayu_sabah"


_DESCRIPTION = """\
Korpus Variasi Bahasa Melayu: Sabah is a language corpus sourced from various folklores in Melayu Sabah dialect.
"""

_CITATION = """\
@misc{melayusabah,
  author = {Hiroki Nomoto},
  title = {Melayu_Sabah},
  year = {2020},
  publisher = {GitHub},
  journal = {GitHub repository},
  howpublished = {\\url{https://github.com/matbahasa/Melayu_Sabah}},
  commit = {90a46c8268412ccc1f29cdcbbd47354474f12d50}
}
"""

_HOMEPAGE = "https://github.com/matbahasa/Melayu_Sabah"


_LANGUAGES = ["msi"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = {
    "sabah201701": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201701.txt",
    "sabah201702": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201702.txt",
    "sabah201901": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201901.txt",
    "sabah201902": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201902.txt",
    "sabah201903": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201903.txt",
    "sabah201904": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201904.txt",
    "sabah201905": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201905.txt",
    "sabah201906": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201906.txt",
    "sabah201907": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201907.txt",
    "sabah201908": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201908.txt",
    "sabah201909": "https://raw.githubusercontent.com/matbahasa/Melayu_Sabah/master/Sabah201909.txt",
}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class MelayuSabah(datasets.GeneratorBasedBuilder):
    """Korpus Variasi Bahasa Melayu:
    Sabah is a language corpus sourced from various folklores in Melayu Sabah dialect."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = [_URLS[key] for key in _URLS.keys()]
        data_path = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_path[0], "split": "train", "other_path": data_path[1:]},
            )
        ]

    def _generate_examples(self, filepath: Path, split: str, other_path: List) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        filepaths = [filepath] + other_path
        data = []
        for filepath in filepaths[:2]:
            with open(filepath, "r") as f:
                sentences = [line.rstrip() for line in f.readlines()]
                sentences = [sentence.split("\t")[-1] for sentence in sentences]
                data.append("\n".join(sentences))

        for filepath in filepaths[2:]:
            with open(filepath, "r") as f:
                data.append([line.rstrip() for line in f.readlines()])

        for id, text in enumerate(data):
            yield id, {"id": id, "text": text}



import pickle
from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{andreaschandra2020,
  author    = {Chandra, Andreas},
  title     = {Indonesian News Dataset},
  year      = {2020},
  howpublished = {Online},
  url       = {https://github.com/andreaschandra/indonesian-news},
  note      = {Accessed: 2024-02-13},
}
"""

_LANGUAGES = ["ind"]

_DATASETNAME = "indonesian_news_dataset"

_DESCRIPTION = """An imbalanced dataset to classify Indonesian News articles.
The dataset contains 5 class labels: bola, news, bisnis, tekno, and otomotif.
The dataset comprises of around 6k train and 2.5k test examples, with the more prevalent classes
(bola and news) having roughly 10x the number of train and test examples than the least prevalent class (otomotif).
"""

_HOMEPAGE = "https://github.com/andreaschandra/indonesian-news"

_LICENSE = Licenses.UNKNOWN.value

_URLS = {
    f"{_DATASETNAME}_train": "https://drive.usercontent.google.com/u/0/uc?id=1wCwPMKSyTciv8I3g9xGdUfEraA1SydG6&export=download",
    f"{_DATASETNAME}_test": "https://drive.usercontent.google.com/u/0/uc?id=1AFW_5KQFW86jlFO16S9bt564Y86WoJjV&export=download",
}

_SUPPORTED_TASKS = [Tasks.TOPIC_MODELING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_TAGS = ["bola", "news", "bisnis", "tekno", "otomotif"]

_LOCAL = False


class IndonesianNewsDataset(datasets.GeneratorBasedBuilder):
    """The dataset contains 5 Indonesian News articles with imbalanced classes"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "news": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(_TAGS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        train_dir = Path(dl_manager.download(_URLS[f"{_DATASETNAME}_train"]))
        test_dir = Path(dl_manager.download(_URLS[f"{_DATASETNAME}_test"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_dir,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_dir,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str):
        """Yields examples as (key, example) tuples."""

        with open(filepath, "rb") as file:
            news_file = pickle.load(file)

        news_list = news_file[0]
        label_list = news_file[1]

        if self.config.schema == "source":
            for idx, (news, label) in enumerate(zip(news_list, label_list)):
                example = {"index": str(idx), "news": news, "label": label}
                yield idx, example
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for idx, (news, label) in enumerate(zip(news_list, label_list)):
                example = {"id": str(idx), "text": news, "label": label}
                yield idx, example
        else:
            raise ValueError(f"Invalid config: {self.config.name}")
